import assign from 'core-js/fn/object/assign'
import find from 'core-js/fn/array/find'
import exportPlugin from '../../lib/exportPlugin'

class VimeoAnalytics {
  constructor(player, metadata) {
    this.player = player
    this.metadata = metadata
    this.mostRecentPosition = 0
    this.isPaused = false
  }

  initialize() {
    const { player } = this
    // Loaded is called by Vimeo immediately after the video gets loaded into the player.
    // We want to call this immediately to set the currentVideoMetadata property to the class instance.
    player.on('loaded', ({ id }) => this.setCurrentVideo(id))
    player.on('play', data => this.trackPlay(data))
    player.on('pause', data => this.trackPause(data))
    player.on('ended', data => this.trackEnded(data))
    player.on('timeupdate', data => this.trackHeartbeat(data))
    player.on('seeked', data => this.trackSeek(data))
  }

  setCurrentVideo(videoId) {
    const { player, metadata } = this
    const currentVideoMetadata = find(metadata, m => m.assetId == videoId)
    if (!currentVideoMetadata) {
      console.warn(
        `Could not locate metadata for Vimeo Video with id ${videoId}`
      )
    } else {
      this.currentVideoMetadata = currentVideoMetadata
    }
    this.videoId = videoId
  }

  trackPlay(data) {
    const { isPaused, currentVideoMetadata } = this
    this.getPlayerProps(data).then(props => {
      if (isPaused) {
        window.analytics.track('Video Playback Resumed', props)
        window.analytics.track(
          'Video Content Started',
          assign(props, currentVideoMetadata)
        )
        this.isPaused = false
      } else {
        window.analytics.track('Video Playback Started', props)
        window.analytics.track(
          'Video Content Started',
          assign(props, currentVideoMetadata)
        )
      }
    })
  }

  trackEnded(data) {
    this.getPlayerProps(data).then(props => {
      window.analytics.track('Video Playback Completed', props)
      window.analytics.track(
        'Video Content Completed',
        assign(props, this.currentVideoMetadata)
      )
    })
  }

  trackHeartbeat(data) {
    // Vimeo provides time updates every 250ms while Segment documents sending heartbeats every 10s.
    // Therefore, we need to do some math to ensure we are not sending 4 heartbeat events when we reach
    // 10 second intervals.
    const mostRecentPosition = Math.floor(this.mostRecentPosition)
    const currentPosition = Math.floor(data.seconds)
    if (
      currentPosition > 9 &&
      currentPosition > mostRecentPosition &&
      currentPosition % 10 === 0
    ) {
      this.getPlayerProps(data).then(props => {
        window.analytics.track(
          'Video Content Playing',
          assign(props, this.currentVideoMetadata)
        )
      })
    }
    this.mostRecentPosition = data.seconds
  }

  trackSeek(data) {
    this.getPlayerProps(data).then(props => {
      window.analytics.track(
        'Video Playback Seek Started',
        assign({}, props, {
          position: this.currentPosition,
          seekPosition: data.seconds
        })
      )
      window.analytics.track('Video Playback Seek Completed', props)
    })
  }

  trackPause(data) {
    this.isPaused = true
    this.mostRecentPosition = data.seconds

    this.getPlayerProps(data).then(props => {
      window.analytics.track('Video Playback Paused', props)
    })
  }

  getPlayerProps(data) {
    const { videoId, player, currentVideoMetadata } = this

    return new Promise((resolve, reject) => {
      const props = {
        totalLength: data.duration,
        sessionId: currentVideoMetadata.sessionId,
        position: data.seconds,
        videoPlayer: 'vimeo'
      }

      player
        .getVolume()
        .then(volume => {
          props.sound = volume * 100
          if (!videoId) {
            return player.getVideoId()
          } else {
            props.videoId = videoId
            resolve(props)
          }
        })
        .then(videoId => {
          this.videoId = props.videoId = videoId
          resolve(props)
        })
        .catch(err => {
          reject(err)
        })
    })
  }
}

export default exportPlugin(VimeoAnalytics)
